# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "읽기 %d분 남음.",
    "(active)": "(활성됨)",
    "Also available in:": "이곳에서도 가능함:",
    "Archive": "저장소",
    "Atom feed": "아톰 피드",
    "Authors": "작성자",
    "Categories": "분류",
    "Comments": "댓글",
    "LANGUAGE": "한국어",
    "Languages:": "언어:",
    "More posts about %s": "%s에 대한 또다른 포스트",
    "Newer posts": "최신 포스트",
    "Next post": "다음 포스트",
    "Next": "다음",
    "No posts found.": "검색된 포스트 없음.",
    "Nothing found.": "검색 결과 없음.",
    "Older posts": "옛날 포스트",
    "Original site": "출처",
    "Posted:": "작성됨:",
    "Posts about %s": "%s에 대한 포스트",
    "Posts by %s": "%s의 작성된 포스트",
    "Posts for year %s": "%s년도 포스트",
    "Posts for {month_day_year}": "{month_day_year}에 작성된 포스트",
    "Posts for {month_year}": "{month_year}에 쓴 포스트",
    "Previous post": "이전 포스트",
    "Previous": "이전",
    "Publication date": "발간일",
    "RSS feed": "RSS 목록",
    "Read in English": "영어로 읽기",
    "Read more": "더 읽기",
    "Skip to main content": "주 콘텐츠로 바로가기",
    "Source": "원문",
    "Subcategories:": "서브 카테고리:",
    "Tags and Categories": "태그와 분류",
    "Tags": "태그",
    "Toggle navigation": "네비게이션 켜고 끄기",
    "Uncategorized": "카테고리 없음",
    "Up": "상위",
    "Updates": "업데이트",
    "Write your page here.": "여기에 페이지를 작성하세요.",
    "Write your post here.": "이곳에 글을 작성하세요.",
    "old posts, page %d": "이전 포스트, 페이지 %d",
    "page %d": "페이지 %d",
    "updated": "업데이트됨",
}
