# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "",
    "(active)": "",
    "Also available in:": "",
    "Archive": "",
    "Atom feed": "",
    "Authors": "",
    "Categories": "",
    "Comments": "Ixfawalen",
    "LANGUAGE": "Tamaziɣt",
    "Languages:": "",
    "More posts about %s": "",
    "Newer posts": "",
    "Next post": "",
    "Next": "",
    "No posts found.": "",
    "Nothing found.": "",
    "Older posts": "",
    "Original site": "",
    "Posted:": "",
    "Posts about %s": "",
    "Posts by %s": "",
    "Posts for year %s": "",
    "Posts for {month_day_year}": "",
    "Posts for {month_year}": "",
    "Previous post": "",
    "Previous": "",
    "Publication date": "",
    "RSS feed": "",
    "Read in English": "Ɣer s tneglizt",
    "Read more": "Ɣer uggar",
    "Skip to main content": "",
    "Source": "Asagem",
    "Subcategories:": "",
    "Tags and Categories": "",
    "Tags": "",
    "Toggle navigation": "",
    "Uncategorized": "",
    "Up": "",
    "Updates": "",
    "Write your page here.": "",
    "Write your post here.": "",
    "old posts, page %d": "",
    "page %d": "",
    "updated": "",
}
